"""Monkey-patch Nox to install packages using nox-poetry.

.. deprecated:: 0.8
   Use :func:`session` instead.

Import this module to monkey-patch Nox.
See :func:`nox_poetry.core.patch` for details.
"""
from nox_poetry.core import patch


patch()



"""Type stubs for nox_poetry.sessions."""
from pathlib import Path
from typing import Any
from typing import Callable
from typing import Dict
from typing import Iterable
from typing import List
from typing import Mapping
from typing import NoReturn
from typing import Optional
from typing import overload
from typing import Sequence
from typing import TypeVar
from typing import Union

import nox.sessions
import nox.virtualenv

Python = Optional[Union[str, Sequence[str], bool]]

class _PoetrySession:
    def install(self, *args: str, **kwargs: Any) -> None: ...
    def installroot(
        self, *, distribution_format: str = ..., extras: Iterable[str] = ...
    ) -> None: ...
    def export_requirements(self) -> Path: ...
    def build_package(self, *, distribution_format: str = ...) -> str: ...

class Session:
    poetry: _PoetrySession
    _session: nox.Session
    def __init__(self, session: nox.Session) -> None: ...
    def install(self, *args: str, **kwargs: Any) -> None: ...
    @property
    def env(self) -> Dict[str, str]: ...
    @property
    def posargs(self) -> List[str]: ...
    @property
    def virtualenv(self) -> nox.virtualenv.ProcessEnv: ...
    @property
    def python(self) -> Python: ...
    @property
    def bin_paths(self) -> Optional[List[str]]: ...
    @property
    def bin(self) -> Optional[str]: ...
    def create_tmp(self) -> str: ...
    @property
    def interactive(self) -> bool: ...
    def chdir(self, dir: str) -> None: ...
    def cd(self, dir: str) -> None: ...
    def run(
        self, *args: str, env: Optional[Mapping[str, str]] = ..., **kwargs: Any
    ) -> Optional[Any]: ...
    def run_always(
        self, *args: str, env: Optional[Mapping[str, str]] = ..., **kwargs: Any
    ) -> Optional[Any]: ...
    def conda_install(
        self, *args: str, auto_offline: bool = ..., **kwargs: Any
    ) -> None: ...
    def notify(self, target: Union[str, nox.sessions.SessionRunner]) -> None: ...
    def log(self, *args: Any, **kwargs: Any) -> None: ...
    def error(self, *args: Any) -> NoReturn: ...
    def skip(self, *args: Any) -> NoReturn: ...

SessionFunction = Callable[..., None]
SessionDecorator = Callable[[SessionFunction], SessionFunction]
@overload
def session(__func: SessionFunction) -> SessionFunction: ...
@overload
def session(
    __func: None = ...,
    python: Python = ...,
    py: Python = ...,
    reuse_venv: Optional[bool] = ...,
    name: Optional[str] = ...,
    venv_backend: Any = ...,
    venv_params: Any = ...,
) -> SessionDecorator: ...



"""Poetry interface."""
from enum import Enum
from pathlib import Path
from typing import List
from typing import Optional

import tomlkit
from nox.sessions import Session


class DistributionFormat(str, Enum):
    """Type of distribution archive for a Python package."""

    WHEEL = "wheel"
    SDIST = "sdist"


class Config:
    """Poetry configuration."""

    def __init__(self, project: Path) -> None:
        """Initialize."""
        path = project / "pyproject.toml"
        text = path.read_text(encoding="utf-8")
        data = tomlkit.parse(text)
        self._config = data["tool"]["poetry"]

    @property
    def name(self) -> str:
        """Return the package name."""
        name = self._config["name"]
        assert isinstance(name, str)  # noqa: S101
        return name

    @property
    def extras(self) -> List[str]:
        """Return the package extras."""
        extras = self._config.get("extras", {})
        assert isinstance(extras, dict) and all(  # noqa: S101
            isinstance(extra, str) for extra in extras
        )
        return list(extras)


class Poetry:
    """Helper class for invoking Poetry inside a Nox session.

    Attributes:
        session: The Session object.
    """

    def __init__(self, session: Session) -> None:
        """Initialize."""
        self.session = session
        self._config: Optional[Config] = None

    @property
    def config(self) -> Config:
        """Return the package configuration."""
        if self._config is None:
            self._config = Config(Path.cwd())
        return self._config

    def export(self) -> str:
        """Export the lock file to requirements format.

        Returns:
            The generated requirements as text.
        """
        output = self.session.run_always(
            "poetry",
            "export",
            "--format=requirements.txt",
            "--dev",
            *[f"--extras={extra}" for extra in self.config.extras],
            "--without-hashes",
            external=True,
            silent=True,
            stderr=None,
        )
        assert isinstance(output, str)  # noqa: S101
        return output

    def build(self, *, format: str) -> str:
        """Build the package.

        The filename of the archive is extracted from the output Poetry writes
        to standard output, which currently looks like this::

           Building foobar (0.1.0)
            - Building wheel
            - Built foobar-0.1.0-py3-none-any.whl

        This is brittle, but it has the advantage that it does not rely on
        assumptions such as having a clean ``dist`` directory, or
        reconstructing the filename from the package metadata. (Poetry does not
        use PEP 440 for version numbers, so this is non-trivial.)

        Args:
            format: The distribution format, either wheel or sdist.

        Returns:
            The basename of the wheel built by Poetry.
        """
        if not isinstance(format, DistributionFormat):
            format = DistributionFormat(format)

        output = self.session.run_always(
            "poetry",
            "build",
            f"--format={format}",
            external=True,
            silent=True,
            stderr=None,
        )
        assert isinstance(output, str)  # noqa: S101
        return output.split()[-1]



"""Core functions.

.. deprecated:: 0.8
   Use :func:`session` instead.
"""
import warnings
from pathlib import Path
from typing import Any
from typing import Iterable
from typing import Optional

import nox.sessions

from nox_poetry.poetry import DistributionFormat
from nox_poetry.sessions import Session


Session_install = nox.sessions.Session.install


def _deprecate(name: str, replacement: Optional[str] = None) -> None:
    message = f"nox_poetry.{name} is deprecated, use @nox_poetry.session instead"
    if replacement is not None:
        message += f" and invoke {replacement}"
    warnings.warn(message, category=FutureWarning, stacklevel=2)


def export_requirements(session: nox.sessions.Session) -> Path:
    """Export a requirements file from Poetry.

    .. deprecated:: 0.8
       Use :func:`session` instead.
    """  # noqa: D
    _deprecate("export_requirements", "session.poetry.export_requirements")
    return Session(session).poetry.export_requirements()


def build_package(session: nox.sessions.Session, *, distribution_format: str) -> str:
    """Build a distribution archive for the package.  # noqa: DAR

    .. deprecated:: 0.8
       Use :func:`session` instead.
    """
    _deprecate("build_package", "session.poetry.build_package")
    return Session(session).poetry.build_package(
        distribution_format=distribution_format
    )


def install(session: nox.sessions.Session, *args: str, **kwargs: Any) -> None:
    """Install packages into a Nox session using Poetry.

    .. deprecated:: 0.8
       Use :func:`session` instead.
    """  # noqa: D
    _deprecate("install", "session.install")
    Session(session).install(*args, **kwargs)


def installroot(
    session: nox.sessions.Session,
    *,  # noqa: DAR
    distribution_format: str,
    extras: Iterable[str] = (),
) -> None:
    """Install the root package into a Nox session using Poetry.

    .. deprecated:: 0.8
       Use :func:`session` instead.
    """
    _deprecate("installroot", "session.poetry.installroot")
    Session(session).poetry.installroot(
        distribution_format=distribution_format, extras=extras
    )


def patch(*, distribution_format: str = DistributionFormat.WHEEL) -> None:
    """Monkey-patch Nox to intercept ``session.install``.

    .. deprecated:: 0.8
       Use :func:`session` instead.

    This function monkey-patches `nox.sessions.Session.install`_ to invoke
    :func:`nox_poetry.install` instead. In addition, the argument ``"."`` is
    replaced by the specified distribution format, or :const:`nox_poetry.WHEEL`
    if none is specified.

    Instead of invoking this function directly, you can simply import
    :mod:`nox_poetry.patch`.

    .. _nox.sessions.Session.install:
       https://nox.thea.codes/en/stable/config.html#nox.sessions.Session.install

    Args:
        distribution_format: The distribution format to use when the ``"."``
            argument is encountered in calls to ``session.install``.
    """
    _deprecate("patch")
    nox.sessions.Session.install = install  # type: ignore[assignment]



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['nox_poetry']

package_data = \
{'': ['*']}

install_requires = \
['nox>=2020.8.22', 'packaging>=20.9', 'tomlkit>=0.7.0,<0.8.0']

setup_kwargs = {
    'name': 'nox-poetry',
    'version': '0.8.4',
    'description': 'nox-poetry',
    'long_description': 'nox-poetry\n==========\n\n|Status| |PyPI| |Python Version| |License|\n\n|Read the Docs| |Tests| |Codecov|\n\n|pre-commit| |Black|\n\n.. |Status| image:: https://badgen.net/badge/status/alpha/d8624d\n   :target: https://badgen.net/badge/status/alpha/d8624d\n   :alt: Project Status\n.. |PyPI| image:: https://img.shields.io/pypi/v/nox-poetry.svg\n   :target: https://pypi.org/project/nox-poetry/\n   :alt: PyPI\n.. |Python Version| image:: https://img.shields.io/pypi/pyversions/nox-poetry\n   :target: https://pypi.org/project/nox-poetry\n   :alt: Python Version\n.. |License| image:: https://img.shields.io/pypi/l/nox-poetry\n   :target: https://opensource.org/licenses/MIT\n   :alt: License\n.. |Read the Docs| image:: https://img.shields.io/readthedocs/nox-poetry/latest.svg?label=Read%20the%20Docs\n   :target: https://nox-poetry.readthedocs.io/\n   :alt: Read the documentation at https://nox-poetry.readthedocs.io/\n.. |Tests| image:: https://github.com/cjolowicz/nox-poetry/workflows/Tests/badge.svg\n   :target: https://github.com/cjolowicz/nox-poetry/actions?workflow=Tests\n   :alt: Tests\n.. |Codecov| image:: https://codecov.io/gh/cjolowicz/nox-poetry/branch/main/graph/badge.svg\n   :target: https://codecov.io/gh/cjolowicz/nox-poetry\n   :alt: Codecov\n.. |pre-commit| image:: https://img.shields.io/badge/pre--commit-enabled-brightgreen?logo=pre-commit&logoColor=white\n   :target: https://github.com/pre-commit/pre-commit\n   :alt: pre-commit\n.. |Black| image:: https://img.shields.io/badge/code%20style-black-000000.svg\n   :target: https://github.com/psf/black\n   :alt: Black\n\n\nUse Poetry_ inside Nox_ sessions\n\nThis package provides a drop-in replacement for the ``nox.session`` decorator,\nand for the ``nox.Session`` object passed to user-defined session functions.\nThis enables ``session.install`` to install packages at the versions specified in the Poetry lock file.\n\n.. code:: python\n\n    from nox_poetry import session\n\n    @session(python=["3.8", "3.9"])\n    def tests(session):\n        session.install("pytest", ".")\n        session.run("pytest")\n\n\n**Disclaimer:** *This project is not affiliated with Nox, and not an official Nox plugin.*\n\n\nInstallation\n------------\n\nInstall ``nox-poetry`` from the Python Package Index:\n\n.. code:: console\n\n   $ pip install nox-poetry\n\n\n**Important:**\nThis package must be installed into the same environment that Nox is run from.\nIf you installed Nox using pipx_,\nuse the following command to install this package into the same environment:\n\n.. code:: console\n\n   $ pipx inject nox nox-poetry\n\n\nUsage\n-----\n\nImport the ``@session`` decorator from ``nox_poetry`` instead of ``nox``.\nThere is nothing else you need to do.\nThe ``session.install`` method automatically honors the Poetry lock file when installing dependencies.\nThis allows you to manage packages used in Nox sessions as development dependencies in Poetry.\n\nThis works because session functions are passed instances of ``nox_poetry.Session``,\na proxy for ``nox.Session`` adding Poetry-related functionality.\nBehind the scenes, nox-poetry uses Poetry to export a `constraints file`_ and build the package.\n\nFor more fine-grained control, additional utilities are available under the ``session.poetry`` attribute:\n\n- ``session.poetry.installroot(distribution_format=[WHEEL|SDIST])``\n- ``session.poetry.build_package(distribution_format=[WHEEL|SDIST])``\n- ``session.poetry.export_requirements()``\n\n\nWhy?\n----\n\nThe example session above performs the following steps:\n\n- Build a wheel from the local package.\n- Install the wheel as well as the ``pytest`` package.\n- Invoke ``pytest`` to run the test suite against the installation.\n\nConsider what would happen in this session\nif we had imported ``@session`` from ``nox`` instead of ``nox_poetry``:\n\n- Package dependencies would only be constrained by the wheel metadata, not by the lock file.\n  In other words, their versions would not be *pinned*.\n- The ``pytest`` dependency would not be constrained at all.\n- Poetry would be installed as a build backend every time.\n\nUnpinned dependencies mean that your checks are not reproducible and deterministic,\nwhich can lead to surprises in Continuous Integration and when collaborating with others.\nYou can solve these issues by declaring ``pytest`` as a development dependency,\nand installing your package and its dependencies using ``poetry install``:\n\n.. code:: python\n\n   @nox.session\n   def tests(session: Session) -> None:\n       """Run the test suite."""\n       session.run("poetry", "install", external=True)\n       session.run("pytest")\n\nUnfortunately, this approach comes with its own set of problems:\n\n- Checks run against an editable installation of your package,\n  i.e. your local copy of the code, instead of the installed wheel your users see.\n  In the best case, any mistakes will still be caught during Continuous Integration.\n  In the worst case, you publish a buggy release because you forgot to commit some changes.\n- The package is installed, as well as all of its core and development dependencies,\n  no matter which tools a session actually runs.\n  Code formatters or linters, for example, don\'t need your package installed at all.\n  Besides being wasteful, it goes against the idea of running checks in isolated environments.\n\n``nox-poetry`` uses a third approach:\n\n- Installations are performed by pip, via the ``session.install`` method.\n- When installing your own package, Poetry is used to build a wheel, which is passed to pip.\n- When installing third-party packages, Poetry is used to export a `constraints file`_,\n  which is passed to pip along with the packages.\n  The constraints file ensures that package versions are pinned by the lock file,\n  without forcing an installation of every listed dependency and sub-dependency.\n\nIn summary, this approach brings the following advantages:\n\n- You can manage tools like ``pytest`` as development dependencies in Poetry.\n- Dependencies are pinned by Poetry\'s lock file, making checks predictable and deterministic.\n- You can run checks against an installed wheel, instead of your local copy of the code.\n- Every tool can run in an isolated environment with minimal dependencies.\n- No need to install your package with all its dependencies if all you need is some linter.\n\n\nContributing\n------------\n\nContributions are very welcome.\nTo learn more, see the `Contributor Guide`_.\n\n\nLicense\n-------\n\n``nox-poetry`` is free and open source software,\ndistributed under the terms of the MIT_ license.\n\n\nIssues\n------\n\nIf you encounter any problems,\nplease `file an issue`_ along with a detailed description.\n\n\nCredits\n-------\n\nThis project was generated from `@cjolowicz`_\'s `Hypermodern Python Cookiecutter`_ template.\n\n\n.. _@cjolowicz: https://github.com/cjolowicz\n.. _Cookiecutter: https://github.com/audreyr/cookiecutter\n.. _Hypermodern Python Cookiecutter: https://github.com/cjolowicz/cookiecutter-hypermodern-python\n.. _MIT: http://opensource.org/licenses/MIT\n.. _Nox: https://nox.thea.codes/\n.. _Poetry: https://python-poetry.org/\n.. _constraints file: https://pip.pypa.io/en/stable/user_guide/#constraints-files\n.. _file an issue: https://github.com/cjolowicz/nox-poetry/issues\n.. _nox.sessions.Session.install: https://nox.thea.codes/en/stable/config.html#nox.sessions.Session.install\n.. _nox.sessions.Session.run: https://nox.thea.codes/en/stable/config.html#nox.sessions.Session.run\n.. _pip install: https://pip.pypa.io/en/stable/reference/pip_install/\n.. _pip: https://pip.pypa.io/\n.. _pipx: https://pipxproject.github.io/pipx/\n.. github-only\n.. _Contributor Guide: CONTRIBUTING.rst\n',
    'author': 'Claudio Jolowicz',
    'author_email': 'mail@claudiojolowicz.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/cjolowicz/nox-poetry',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.6.1,<4.0.0',
}


setup(**setup_kwargs)



"""Using Poetry in Nox sessions.

This package provides a drop-in replacement for the :func:`session` decorator,
and for the :class:`Session` object passed to user-defined session functions.
This enables :meth:`session.install
<nox_poetry.sessions._PoetrySession.install>` to install packages at the
versions specified in the Poetry lock file.

Example:
    >>> @session(python=["3.8", "3.9"])
    ... def tests(session: Session) -> None:
    ...     session.install("pytest", ".")
    ...     session.run("pytest")

It also provides helper functions that allow more fine-grained control:

- :meth:`session.poetry.installroot
  <nox_poetry.sessions._PoetrySession.installroot>`
- :meth:`session.poetry.build_package
  <nox_poetry.sessions._PoetrySession.build_package>`
- :meth:`session.poetry.export_requirements
  <nox_poetry.sessions._PoetrySession.export_requirements>`

Two constants are defined to specify the format for distribution archives:

- :const:`WHEEL`
- :const:`SDIST`
"""
from nox_poetry.core import build_package
from nox_poetry.core import export_requirements
from nox_poetry.core import install
from nox_poetry.core import installroot
from nox_poetry.poetry import DistributionFormat
from nox_poetry.sessions import Session
from nox_poetry.sessions import session


#: A wheel archive.
WHEEL: str = DistributionFormat.WHEEL

#: A source archive.
SDIST: str = DistributionFormat.SDIST

__all__ = [
    "build_package",
    "export_requirements",
    "install",
    "installroot",
    "Session",
    "session",
    "SDIST",
    "WHEEL",
]



"""Replacements for the ``nox.session`` decorator and the ``nox.Session`` class."""
import functools
import hashlib
import re
from pathlib import Path
from typing import Any
from typing import Iterable
from typing import Iterator
from typing import Optional
from typing import Tuple

import nox
from packaging.requirements import InvalidRequirement
from packaging.requirements import Requirement

from nox_poetry.poetry import DistributionFormat
from nox_poetry.poetry import Poetry


def session(*args: Any, **kwargs: Any) -> Any:
    """Drop-in replacement for the :func:`nox.session` decorator.

    Use this decorator instead of ``@nox.session``. Session functions are passed
    :class:`Session` instead of :class:`nox.sessions.Session`; otherwise, the
    decorators work exactly the same.

    Args:
        args: Positional arguments are forwarded to ``nox.session``.
        kwargs: Keyword arguments are forwarded to ``nox.session``.

    Returns:
        The decorated session function.
    """
    if not args:
        return functools.partial(session, **kwargs)

    [function] = args

    @functools.wraps(function)
    def wrapper(session: nox.Session, *_args, **_kwargs) -> None:
        proxy = Session(session)
        function(proxy, *_args, **_kwargs)

    return nox.session(wrapper, **kwargs)  # type: ignore[call-overload]


_EXTRAS_PATTERN = re.compile(r"^(.+)(\[[^\]]+\])$")


def _split_extras(arg: str) -> Tuple[str, Optional[str]]:
    # From ``pip._internal.req.constructors._strip_extras``
    match = _EXTRAS_PATTERN.match(arg)
    if match:
        return match.group(1), match.group(2)
    return arg, None


def to_constraint(requirement_string: str, line: int) -> Optional[str]:
    """Convert requirement to constraint."""
    if any(
        requirement_string.startswith(prefix)
        for prefix in ("-", "file://", "git+https://", "http://", "https://")
    ):
        return None

    try:
        requirement = Requirement(requirement_string)
    except InvalidRequirement as error:
        raise RuntimeError(f"line {line}: {requirement_string!r}: {error}")

    if not (requirement.name and requirement.specifier):
        return None

    constraint = f"{requirement.name}{requirement.specifier}"
    return f"{constraint}; {requirement.marker}" if requirement.marker else constraint


def to_constraints(requirements: str) -> str:
    """Convert requirements to constraints."""

    def _to_constraints() -> Iterator[str]:
        lines = requirements.splitlines()
        for line, requirement in enumerate(lines, start=1):
            if requirement.strip():
                constraint = to_constraint(requirement, line)
                if constraint is not None:
                    yield constraint

    return "\n".join(_to_constraints())


class _PoetrySession:
    """Poetry-related utilities for session functions."""

    def __init__(self, session: nox.Session) -> None:
        """Initialize."""
        self.session = session
        self.poetry = Poetry(session)

    def install(self, *args: str, **kwargs: Any) -> None:
        """Install packages into a Nox session using Poetry.

        This function installs packages into the session's virtual environment. It
        is a wrapper for :meth:`nox.sessions.Session.install`, whose positional
        arguments are command-line arguments for :ref:`pip install`, and whose keyword
        arguments are the same as those for :meth:`nox.sessions.Session.run`.

        If a positional argument is ".", a wheel is built using
        :meth:`build_package`, and the argument is replaced with the file URL
        returned by that function. Otherwise, the argument is forwarded unchanged.

        In addition, a :ref:`constraints file <Constraints Files>` is generated
        for the package dependencies using :meth:`export_requirements`, and
        passed to ``pip install`` via its ``--constraint`` option. This ensures
        that any package installed will be at the version specified in Poetry's
        lock file.

        Args:
            args: Command-line arguments for ``pip install``.
            kwargs: Keyword-arguments for ``session.install``. These are the same
                as those for :meth:`nox.sessions.Session.run`.
        """
        from nox_poetry.core import Session_install

        args_extras = [_split_extras(arg) for arg in args]

        if "." in [arg for arg, _ in args_extras]:
            package = self.build_package()

            def rewrite(arg: str, extras: Optional[str]) -> str:
                if arg != ".":
                    return arg if extras is None else arg + extras

                if extras is None:
                    return package

                name = self.poetry.config.name
                return f"{name}{extras} @ {package}"

            args = tuple(rewrite(arg, extras) for arg, extras in args_extras)

            self.session.run_always("pip", "uninstall", "--yes", package, silent=True)

        requirements = self.export_requirements()
        Session_install(self.session, f"--constraint={requirements}", *args, **kwargs)

    def installroot(
        self,
        *,
        distribution_format: str = DistributionFormat.WHEEL,
        extras: Iterable[str] = (),
    ) -> None:
        """Install the root package into a Nox session using Poetry.

        This function installs the package located in the current directory into the
        session's virtual environment.

        A :ref:`constraints file <Constraints Files>` is generated for the
        package dependencies using :meth:`export_requirements`, and passed to
        :ref:`pip install` via its ``--constraint`` option. This ensures that
        core dependencies are installed using the versions specified in Poetry's
        lock file.

        Args:
            distribution_format: The distribution format, either wheel or sdist.
            extras: Extras to install for the package.
        """
        from nox_poetry.core import Session_install

        package = self.build_package(distribution_format=distribution_format)
        requirements = self.export_requirements()

        self.session.run_always("pip", "uninstall", "--yes", package, silent=True)

        suffix = ",".join(extras)
        if suffix.strip():
            suffix = suffix.join("[]")
            name = self.poetry.config.name
            package = f"{name}{suffix} @ {package}"

        Session_install(self.session, f"--constraint={requirements}", package)

    def export_requirements(self) -> Path:
        """Export a requirements file from Poetry.

        This function uses `poetry export`_ to generate a :ref:`requirements
        file <Requirements Files>` containing the project dependencies at the
        versions specified in ``poetry.lock``. The requirements file includes
        both core and development dependencies.

        The requirements file is stored in a per-session temporary directory,
        together with a hash digest over ``poetry.lock`` to avoid generating the
        file when the dependencies have not changed since the last run.

        .. _poetry export: https://python-poetry.org/docs/cli/#export

        Returns:
            The path to the requirements file.
        """
        tmpdir = Path(self.session.virtualenv.location) / "tmp"
        tmpdir.mkdir(exist_ok=True)

        path = tmpdir / "requirements.txt"
        hashfile = tmpdir / f"{path.name}.hash"

        lockdata = Path("poetry.lock").read_bytes()
        digest = hashlib.blake2b(lockdata).hexdigest()

        if not hashfile.is_file() or hashfile.read_text() != digest:
            constraints = to_constraints(self.poetry.export())
            path.write_text(constraints)
            hashfile.write_text(digest)

        return path

    def build_package(
        self, *, distribution_format: str = DistributionFormat.WHEEL
    ) -> str:
        """Build a distribution archive for the package.

        This function uses `poetry build`_ to build a wheel or sdist archive for
        the local package, as specified via the ``distribution_format`` parameter.
        It returns a file URL with the absolute path to the built archive.

        .. _poetry build: https://python-poetry.org/docs/cli/#export

        Args:
            distribution_format: The distribution format, either wheel or sdist.

        Returns:
            The file URL for the distribution package.
        """
        wheel = Path("dist") / self.poetry.build(format=distribution_format)
        url = wheel.resolve().as_uri()

        if DistributionFormat(distribution_format) is DistributionFormat.SDIST:
            url += f"#egg={self.poetry.config.name}"

        return url


class _SessionProxy:
    """Proxy for :class:`nox.sessions.Session`."""

    def __init__(self, session: nox.Session) -> None:
        """Initialize."""
        self._session = session

    def __getattr__(self, name: str) -> Any:
        """Delegate attribute access to nox.Session."""
        return getattr(self._session, name)


class Session(_SessionProxy):
    """Proxy for :class:`nox.sessions.Session`, passed to session functions.

    This class overrides :meth:`session.install
    <nox_poetry.sessions._PoetrySession.install>`, and provides Poetry-related
    utilities:

    - :meth:`Session.poetry.installroot
      <nox_poetry.sessions._PoetrySession.installroot>`
    - :meth:`Session.poetry.build_package
      <nox_poetry.sessions._PoetrySession.build_package>`
    - :meth:`Session.poetry.export_requirements
      <nox_poetry.sessions._PoetrySession.export_requirements>`
    """

    def __init__(self, session: nox.Session) -> None:
        """Initialize."""
        super().__init__(session)
        self.poetry = _PoetrySession(session)

    def install(self, *args: str, **kwargs: Any) -> None:
        """Install packages into a Nox session using Poetry."""
        return self.poetry.install(*args, **kwargs)
